"""Add idna_subject column into whois record

Revision ID: 459a0d7b8f09
Revises: 45713fea8097
Create Date: 2020-12-07 12:37:52.018637

"""
import sqlalchemy as sa
from alembic import op

# pylint: skip-file

# revision identifiers, used by Alembic.
revision = "459a0d7b8f09"
down_revision = "45713fea8097"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "pyfunceble_whois_record", sa.Column("idna_subject", sa.Text(), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("pyfunceble_whois_record", "idna_subject")
    # ### end Alembic commands ###
